﻿using System;
using Luticate2.Auth.Auth.Business;
using Luticate2.Auth.Auth.DataAccess;
using Luticate2.Auth.ConsoleSample.Commands;
using Luticate2.Auth.Utils.Business.Converters;
using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.DependencyInjection;
using NClap.Metadata;
using NClap.Repl;

namespace Luticate2.Auth.ConsoleSample
{
    enum CommandsEnum
    {
        [Command(typeof(ListCommand), Description = "Lists Luticate2 items", LongName = "ls", ShortName = "l")]
        ListItems,

        [Command(typeof(CreateCommand), Description = "Creates Luticate2 items", LongName = "create", ShortName = "c")]
        CreateItems,

        [Command(typeof(ExitCommand), Description = "Exits the shell")]
        Exit
    }
    
    class Program
    {
        public static IServiceProvider ServiceProvider;
        
        static void Main(string[] args)
        {
            IServiceCollection services = new ServiceCollection();

            services.AddLuObjectConverterDescriptors();
            services.AddLuObjectConverters();
            
            services.AddScoped<LuGroupsBusiness>();

            services.AddDbContext<Luticate2DbContext>(options =>
            {
                options.UseNpgsql(@"Host=localhost;Database=luticate2;Username=dev;Password=dev");
                options.UseInternalServiceProvider(new ServiceCollection()
                    .AddEntityFrameworkNpgsql()
                    .BuildServiceProvider());
            }, ServiceLifetime.Transient);

            ServiceProvider = services.BuildServiceProvider();
            
            var loop = new Loop(typeof(CommandsEnum));
            loop.Execute();
        }
    }
}